package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantOutput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.cloud.space.service.ISpaceOptimizeLongCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.enums.BusinessError;
import pps.core.space.mapper.*;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 长期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 14:14
 */
@XService
public class SpaceOptimizeLongCloudServiceImpl implements ISpaceOptimizeLongCloudService {

    /**
     * 长期间开优化Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeLongJob(XContext context) {
        //取生效中的基础间开
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        List<SpaceInstitutionDetailEnt> detailEntList = detailMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ZERO)
                .apply(" institution_end_date >= CURDATE() ")
        );
        if (CollUtil.isEmpty(detailEntList)) {
            //未发现可优化基础间开制度
            return XServiceResult.error(context, BusinessError.DidNotFindSpace);
        }
        Set<String> lineIds = new HashSet<>(32);
        Set<String> institutionIds = new HashSet<>(32);
        for (SpaceInstitutionDetailEnt spaceInstitutionDetailEnt : detailEntList) {
            lineIds.add(spaceInstitutionDetailEnt.getLineId());
            institutionIds.add(spaceInstitutionDetailEnt.getId());
        }
        //取生效中间开的所有井口
        SpaceInstitutionWellheadViewMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
        List<SpaceInstitutionWellheadView> spaceWellheadList = wellheadMapper.selectListByInstitutionIds(institutionIds);
        if (CollUtil.isEmpty(spaceWellheadList)) {
            //未发现可优化井口
            return XServiceResult.error(context, BusinessError.DidNotFindWellhead);
        }
        this.setServiceRatingForSpaceWellheadList(context, spaceWellheadList);
        //取线路关联的所有光伏电站ID
        List<DynamicQueryBasePowerLinePlantOutput> plantList = this.getPowerLinePlantListByLineIds(context, lineIds);
        if (CollUtil.isEmpty(plantList)) {
            //未发现可用光伏电站
            return XServiceResult.error(context, BusinessError.DidNotFindPlant);
        }
        //取当前时间
        DateTime date = DateUtil.date();
        int year = date.year();
        int monthNum = date.month();
        String month;
        if (monthNum < BusinessConstant.TEN) {
            month = "0" + monthNum;
        } else {
            month = String.valueOf(monthNum);
        }
        SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
        DateTime dateFlag = DateUtil.parse("1970-01-02 00:00:00", "yyyy-mm-dd HH:mm:ss");
        List<SpaceOptimizeLongPeriodView> longPeriodList = new ArrayList<>(32);
        List<SpaceOptimizeLongWellheadView> longWellheadList = new ArrayList<>(64);
        List<SpaceOptimizeLongDurationView> longDurationList = new ArrayList<>(128);
        String executionCycleForMonth = BaseUtils.getExecutionCycleForMonth(date);
        DateTime optimizeDeadline = DateUtil.endOfMonth(date);
        //优化
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            String detailId = detail.getId();
            String lineId = detail.getLineId();
            Integer startInterval = 0;
            //创建记录
            String longPeriodId = this.createOptimizeLongPeriod(longPeriodList, detailId, lineId, executionCycleForMonth, optimizeDeadline);
            switch (detail.getGridTypeKey()) {
                //并网型优化
                case "1":
                    //获取当前制度对应的光伏预测数据列表
                    List<DynamicQueryPlantPredictedPowerOutput> avgPowerList = this.getAveragePowerGenerationListByPlantIds(context,
                            DynamicQueryPlantPredictedPowerInput.builder()
                                    .plantIds(
                                            plantList.stream()
                                                    .filter(p -> StringUtils.equals(lineId, p.getLineId()))
                                                    .map(DynamicQueryBasePowerLinePlantOutput::getPlantId)
                                                    .collect(Collectors.toList())
                                    )
                                    .yearTime(String.valueOf(year))
                                    .monthTime(month)
                                    .build()
                    );
                    //取光伏出力峰值
                    BigDecimal powerMax = avgPowerList.stream()
                            .map(DynamicQueryPlantPredictedPowerOutput::getPower)
                            .max(BigDecimal::compareTo)
                            .orElse(BigDecimal.ZERO);
                    //取当前制度下井口的总功率
                    BigDecimal wellheadTotalPower = spaceWellheadList.stream()
                            .filter(w -> StringUtils.equals(detailId, w.getInstitutionId()))
                            .map(SpaceInstitutionWellheadView::getServiceRating)
                            .reduce(BigDecimal.ZERO, BigDecimal::add);
                    //根据类型过滤井口:大间开,连抽井不优化
                    List<SpaceInstitutionWellheadView> wellheadViewList = spaceWellheadList.stream()
                            .filter(w -> StringUtils.equals(detailId, w.getInstitutionId()) &&
                                    StringUtils.equals("INTERVAL", w.getRunTypeKey()) &&
                                    StringUtils.equals("0", w.getIntervalTypeKey()))
                            .sorted(Comparator.comparing(SpaceInstitutionWellheadView::getStartSeq))
                            .collect(Collectors.toList());
                    if (CollUtil.isEmpty(wellheadViewList)) {
                        continue;
                    }
                    //通过间开ID和井口ID查所有井口时段配置
                    List<SpaceInstitutionDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDurationEnt>()
                            .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                            .eq(SpaceInstitutionDurationEnt::getInstitutionId, detailId)
                            .orderByAsc(SpaceInstitutionDurationEnt::getOpenWellTime)
                    );
                    if (CollUtil.isEmpty(durationList)) {
                        //没有设置时间段,无法优化
                        continue;
                    }
                    Map<String, List<SpaceInstitutionDurationEnt>> durationMap = durationList.stream()
                            .collect(Collectors.groupingBy(SpaceInstitutionDurationEnt::getWellheadId));
                    int compare = powerMax.compareTo(wellheadTotalPower);
                    //光伏出力峰值大于等于井口总功率:绿电消纳优先策略
                    if (compare >= BusinessConstant.ZERO) {
                        //---------------------------------井口优化---------------------------------
                        int between = 0;
                        //判断是否第一口井
                        boolean isFirstWellhead;
                        //第一口井启动时间
                        DateTime firstStartTime = null;
                        //井口累积运行总功率
                        BigDecimal totalOperatingPower = BigDecimal.ZERO;
                        for (int w = 0, wellheadSize = wellheadViewList.size(); w < wellheadSize; w++) {
                            SpaceInstitutionWellheadView wellhead = wellheadViewList.get(w);
                            String wellheadId = wellhead.getWellheadId();
                            String recordId = this.createOptimizeLongWellhead(longWellheadList, longPeriodId, wellheadId, wellhead.getWellNumber());
                            //取当前井口最大发电量
                            BigDecimal serviceRating = wellhead.getServiceRating();
                            List<SpaceInstitutionDurationEnt> durationConfigList = durationMap.get(wellhead);
                            if (CollUtil.isEmpty(durationConfigList)) {
                                //没有设置时间段,无法优化
                                continue;
                            }
                            if (w == 0) {
                                //第一个井口
                                isFirstWellhead = true;
                                totalOperatingPower.add(serviceRating);
                            } else {
                                //累加
                                totalOperatingPower = totalOperatingPower.add(serviceRating);
                                startInterval = startInterval + detail.getStartInterval();
                                isFirstWellhead = false;
                            }
                            for (int d = 0, durationSize = durationConfigList.size(); d < durationSize; d++) {
                                SpaceInstitutionDurationEnt duration = durationConfigList.get(d);
                                String openWellTime = duration.getOpenWellTime();
                                DateTime startTime = DateUtil.parse(openWellTime + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                DateTime endTime = DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                //第一次启动
                                if (d == 0) {
                                    //第一口井的启动时间
                                    if (isFirstWellhead) {
                                        firstStartTime = startTime;
                                        //计算关井时间
                                        int endIndex = 0;
                                        for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                                            DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                                            if (DateUtil.parse(predictedPower.getHourTime() + predictedPower.getMinTime(), BusinessConstant.TIME_FORMAT).compareTo(startTime) < 0) {
                                                continue;
                                            }
                                            //当日时间段平均光伏出力>第一口井运行负荷时,该时间为第一口井运行时间
                                            if (predictedPower.getPower().compareTo(serviceRating) >= BusinessConstant.ZERO) {
                                                endIndex = a;
                                                break;
                                            }
                                        }
                                        DynamicQueryPlantPredictedPowerOutput end = avgPowerList.get(endIndex);
                                        String endTimeString = end.getHourTime() + end.getMinTime();
                                        this.createOptimizeLongDuration(longDurationList, duration, longPeriodId, recordId, wellheadId, null,
                                                openWellTime, StringUtils.substringBeforeLast(endTimeString, BusinessConstant.INITIALIZATION_SECOND)
                                        );
                                        //计算
                                        DateTime endTimeOptimize = DateUtil.parse(endTimeString, BusinessConstant.TIME_FORMAT);
                                        //取时间间隔(分钟)
                                        between = (int) endTimeOptimize.between(endTime, DateUnit.MINUTE);
                                        if (endTimeOptimize.compareTo(endTime) < 0) {
                                            between = ~between + 1;
                                        }
                                    }
                                    //其它井口的第一次启动时间
                                    else {
                                        DateTime startTimeOffset = firstStartTime.offset(DateField.MINUTE, startInterval);
                                        int startIndex = -1;
                                        BigDecimal add = serviceRating.add(totalOperatingPower);
                                        for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                                            DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                                            if (DateUtil.parse(predictedPower.getHourTime() + predictedPower.getMinTime(), BusinessConstant.TIME_FORMAT)
                                                    .compareTo(startTimeOffset) >= 0) {
                                                //判断第一口井启动时间+启动间隔时日平均光伏出力-前两口井的运行功率是否为正数
                                                if (predictedPower.getPower().compareTo(add) >= 0) {
                                                    //确定第二口井第一次开井时间为第一口井启动时间+启动间隔
                                                    startIndex = a;
                                                    break;
                                                }
                                            }
                                        }
                                        if (startIndex > -1) {
                                            DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(startIndex);
                                            DateTime startTimeOptimize = DateUtil.parse(start.getHourTime() + start.getMinTime(), BusinessConstant.TIME_FORMAT);
                                            //计算未优化启动间隔
                                            int openDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                                            DateTime endTimeOptimize = startTimeOptimize.offset(DateField.MINUTE, openDuration);
                                            String endTimeString;
                                            if (endTimeOptimize.compareTo(dateFlag) > 0) {
                                                endTimeString = BusinessConstant.END_OF_DAY_TIME;
                                            } else {
                                                endTimeString = endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT);
                                            }
                                            this.createOptimizeLongDuration(longDurationList, duration, longPeriodId, recordId, wellheadId, null,
                                                    openWellTime, endTimeString
                                            );
                                            //取时间间隔(分钟)
                                            between = (int) endTimeOptimize.between(endTime, DateUnit.MINUTE);
                                            if (endTimeOptimize.compareTo(endTime) < 0) {
                                                between = ~between + 1;
                                            }
                                        } else {
                                            //无法优化
                                        }
                                    }
                                } else {
                                    DateTime offset = startTime.offset(DateField.MINUTE, between);
                                    if (offset.compareTo(dateFlag) > 0) {
                                        //如果时间超过当天,舍弃
                                        continue;
                                    }
                                    DateTime endDate = DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT)
                                            .offset(DateField.MINUTE, between);
                                    String closeWellTime;
                                    if (endDate.compareTo(dateFlag) > 0) {
                                        closeWellTime = BusinessConstant.END_OF_DAY_TIME;
                                    } else {
                                        closeWellTime = endDate.toString(BusinessConstant.MINUTES_FORMAT);
                                    }
                                    //计算偏移
                                    this.createOptimizeLongDuration(longDurationList, duration, longPeriodId, recordId, wellheadId, null,
                                            offset.toString(BusinessConstant.MINUTES_FORMAT), closeWellTime
                                    );
                                }
                            }
                        }
                    } else {
                        //遍历井口,按发电功率大于等于光伏出力峰值条件分组
                        Map<Boolean, List<SpaceInstitutionWellheadView>> collect = spaceWellheadList.stream()
                                .filter(w ->
                                        StringUtils.equals(detailId, w.getInstitutionId())
                                )
                                .collect(Collectors.partitioningBy(w -> powerMax.compareTo(w.getServiceRating()) >= BusinessConstant.ZERO));

                        List<SpaceInstitutionWellheadView> lowWellheadList = collect.get(false);
                        int size = spaceWellheadList.size();
                        int lowWellheadListSize = lowWellheadList.size();
                        //光伏出力峰值<任何一口井的运行功率:消峰平谷策略
                        if (size == lowWellheadListSize) {
                            //---------------------------------井口优化---------------------------------
                            //通过线路ID和月份获取市电峰谷策略明细配置
                            List<GetBasePriceStrategyDetailOutput> strategyDetailList = ServiceUtil.getStrategyDetailList(context,
                                    GetBasePriceStrategyDetailInput.builder()
                                            .lineId(lineId)
                                            .strategyMonth(String.valueOf(monthNum))
                                            .build()
                            );
                            if (CollUtil.isEmpty(strategyDetailList)) {
                                //没有配置,不优化
                                continue;
                            }
                            //获取第一段谷电阶段的开始时间为第一口井的开井时间
                            Optional<GetBasePriceStrategyDetailOutput> low = strategyDetailList.stream()
                                    .filter(s -> !StringUtils.isAnyBlank(s.getEndTime(), s.getStartTime()) && StringUtils.equals("LOW", s.getPeriodTypeKey()))
                                    .findFirst();
                            if (!low.isPresent()) {
                                //没有谷电,不优化
                                continue;
                            }
                            GetBasePriceStrategyDetailOutput strategyDetailOutput = low.get();
                            //第一口井启动时间
                            DateTime firstStartTime = DateUtil.parse(strategyDetailOutput.getStartTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                            DateTime firstEndTime = DateUtil.parse(strategyDetailOutput.getEndTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                            //判断是否第一口井
                            boolean isFirstWellhead;
                            //优化时间间隔
                            int between = 0;
                            for (int w = 0, wellheadSize = wellheadViewList.size(); w < wellheadSize; w++) {
                                SpaceInstitutionWellheadView wellhead = wellheadViewList.get(w);
                                String wellheadId = wellhead.getWellheadId();
                                String recordId = this.createOptimizeLongWellhead(longWellheadList, longPeriodId, wellheadId, wellhead.getWellNumber());
                                List<SpaceInstitutionDurationEnt> durationConfigList = durationMap.get(wellhead);
                                if (CollUtil.isEmpty(durationConfigList)) {
                                    //没有设置时间段,无法优化
                                    continue;
                                }
                                if (w == 0) {
                                    //第一个井口
                                    isFirstWellhead = true;
                                } else {
                                    //计算启动间隔
                                    startInterval = startInterval + detail.getStartInterval();
                                    isFirstWellhead = false;
                                }
                                for (int d = 0, durationSize = durationConfigList.size(); d < durationSize; d++) {
                                    SpaceInstitutionDurationEnt duration = durationConfigList.get(d);
                                    String openWellTime = duration.getOpenWellTime();
                                    DateTime startTime = DateUtil.parse(openWellTime + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                    DateTime endTime = DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                                    //第一次启动
                                    if (d == 0) {
                                        //第一口井的启动时间
                                        if (isFirstWellhead) {
                                            //计算优化后的时间差优化后的第一口井开井时间-优化前第一次开井时间
                                            //取时间间隔(分钟)
                                            between = (int) firstEndTime.between(endTime, DateUnit.MINUTE);
                                            if (firstEndTime.compareTo(endTime) < 0) {
                                                between = ~between + 1;
                                            }
                                            this.createOptimizeLongDuration(longDurationList, duration, longPeriodId, recordId, wellheadId, null,
                                                    strategyDetailOutput.getStartTime(), strategyDetailOutput.getEndTime()
                                            );
                                        }//其它井口的第一次启动时间
                                        else {
                                            //其它井启动时间:第一口井启动时间+启动间隔
                                            DateTime startTimeOffset = firstStartTime.offset(DateField.MINUTE, startInterval);
                                            DateTime endTimeOffset = firstEndTime.offset(DateField.MINUTE, startInterval);
                                            //取时间间隔(分钟)
                                            between = (int) firstEndTime.between(endTime, DateUnit.MINUTE);
                                            if (firstEndTime.compareTo(endTime) < 0) {
                                                between = ~between + 1;
                                            }
                                            this.createOptimizeLongDuration(longDurationList, duration, longPeriodId, recordId, wellheadId, null,
                                                    startTimeOffset.toString(BusinessConstant.MINUTES_FORMAT), endTimeOffset.toString(BusinessConstant.MINUTES_FORMAT)
                                            );
                                        }
                                    } else {
                                        DateTime offset = startTime.offset(DateField.MINUTE, between);
                                        if (offset.compareTo(dateFlag) > 0) {
                                            //如果时间超过当天,舍弃
                                            continue;
                                        }
                                        DateTime endDate = DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT)
                                                .offset(DateField.MINUTE, between);
                                        String closeWellTime;
                                        if (endDate.compareTo(dateFlag) > 0) {
                                            closeWellTime = BusinessConstant.END_OF_DAY_TIME;
                                        } else {
                                            closeWellTime = endDate.toString(BusinessConstant.MINUTES_FORMAT);
                                        }
                                        //计算偏移
                                        this.createOptimizeLongDuration(longDurationList, duration, longPeriodId, recordId, wellheadId, null,
                                                offset.toString(BusinessConstant.MINUTES_FORMAT), closeWellTime
                                        );
                                    }
                                }
                            }
                        }
                        //光伏出力峰值>=线路哪部分井口运行功率:满足的部分井口采用绿电消纳优先,不满足的井采用消峰平谷
                        else {
                            //---------------------------------井口优化---------------------------------
                            List<SpaceInstitutionWellheadView> highWellheadList = collect.get(true);


                        }
                    }
                    break;
                //离网型优化
                case "0":
                    break;
                default:
                    //电网类型不存在
            }
        }
        //开启事务
        return XTransactionHelper.begin(context, () -> {
            int size;
            if (CollUtil.isNotEmpty(longPeriodList)) {
                SpaceOptimizeLongPeriodViewMapper longPeriodViewMapper = context.getBean(SpaceOptimizeLongPeriodViewMapper.class);
                size = longPeriodList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeLongPeriodView>> subList = BaseUtils.getSubList(longPeriodList);
                    subList.forEach(b -> longPeriodViewMapper.batchInsertList(b));
                } else {
                    longPeriodViewMapper.batchInsertList(longPeriodList);
                }
            }
            if (CollUtil.isNotEmpty(longWellheadList)) {
                SpaceOptimizeLongWellheadViewMapper longWellheadViewMapper = context.getBean(SpaceOptimizeLongWellheadViewMapper.class);
                size = longWellheadList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeLongWellheadView>> subList = BaseUtils.getSubList(longWellheadList);
                    subList.forEach(b -> longWellheadViewMapper.batchInsertList(b));
                } else {
                    longWellheadViewMapper.batchInsertList(longWellheadList);
                }
            }
            if (CollUtil.isNotEmpty(longDurationList)) {
                SpaceOptimizeLongDurationViewMapper longDurationViewMapper = context.getBean(SpaceOptimizeLongDurationViewMapper.class);
                size = longDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeLongDurationView>> subList = BaseUtils.getSubList(longDurationList);
                    subList.forEach(b -> longDurationViewMapper.batchInsertList(b));
                } else {
                    longDurationViewMapper.batchInsertList(longDurationList);
                }
            }
            return XServiceResult.OK;
        });
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 创建长期优化信息
     *
     * @param longPeriodList         长期清单
     * @param detailId               详细信息id
     * @param lineId                 线路id
     * @param executionCycleForMonth 月执行周期
     * @param optimizeDeadline       优化截止日期
     * @return {@link String}
     */
    private String createOptimizeLongPeriod(List<SpaceOptimizeLongPeriodView> longPeriodList, String detailId, String lineId, String executionCycleForMonth, Date optimizeDeadline) {
        SpaceOptimizeLongPeriodView longPeriod = new SpaceOptimizeLongPeriodView();
        BaseUtils.setBaseModelDefaultForJob(longPeriod);
        longPeriod.setInstitutionId(detailId);
        longPeriod.setLineId(lineId);
        longPeriod.setExecutionCycle(executionCycleForMonth);
        longPeriod.setOptimizeDeadline(optimizeDeadline);
        longPeriodList.add(longPeriod);
        return longPeriod.getId();
    }

    /**
     * 创建长期优化井口信息
     *
     * @param longWellheadList 长井口列表
     * @param longPeriodId     长周期id
     * @param wellheadId       井口id
     * @param wellNumber       井号
     * @return {@link String}
     */
    private String createOptimizeLongWellhead(List<SpaceOptimizeLongWellheadView> longWellheadList, String longPeriodId, String wellheadId, String wellNumber) {
        SpaceOptimizeLongWellheadView wellheadView = new SpaceOptimizeLongWellheadView();
        BaseUtils.setBaseModelDefaultForJob(wellheadView);
        wellheadView.setLongPeriodId(longPeriodId);
        wellheadView.setWellheadId(wellheadId);
        wellheadView.setWellNumber(wellNumber);
        longWellheadList.add(wellheadView);
        return wellheadView.getId();
    }

    /**
     * 创建长期优化井口区间配置信息
     *
     * @param longDurationList  长期清单
     * @param durationEnt       持续时间ent
     * @param longPeriodId      长周期id
     * @param recordId          记录id
     * @param wellheadId        井口id
     * @param generationTypeKey 生成类型密钥
     * @param openWellTime      开井时间
     * @param endTimeString     结束时间字符串
     */
    private void createOptimizeLongDuration(List<SpaceOptimizeLongDurationView> longDurationList, SpaceInstitutionDurationEnt durationEnt, String longPeriodId,
                                            String recordId, String wellheadId, String generationTypeKey, String openWellTime, String endTimeString) {
        SpaceOptimizeLongDurationView durationView = new SpaceOptimizeLongDurationView();
        BaseUtils.setBaseModelDefaultForJob(durationView);
        durationView.setRecordId(recordId);
        durationView.setLongPeriodId(longPeriodId);
        durationView.setWellheadId(wellheadId);
        durationView.setGenerationTypeKey(generationTypeKey);
        durationView.setIsOptimize(BusinessConstant.ZERO);
        durationView.setOpenWellTime(openWellTime);
        durationView.setCloseWellTime(endTimeString);
        longDurationList.add(durationView);
        SpaceOptimizeLongDurationView durationViewUnOptimize = new SpaceOptimizeLongDurationView();
        BaseUtils.setBaseModelDefaultForJob(durationViewUnOptimize);
        durationViewUnOptimize.setRecordId(recordId);
        durationViewUnOptimize.setLongPeriodId(longPeriodId);
        durationViewUnOptimize.setWellheadId(wellheadId);
        durationViewUnOptimize.setGenerationTypeKey(generationTypeKey);
        durationViewUnOptimize.setIsOptimize(BusinessConstant.ONE);
        durationViewUnOptimize.setOpenWellTime(durationEnt.getOpenWellTime());
        durationViewUnOptimize.setCloseWellTime(durationEnt.getCloseWellTime());
        longDurationList.add(durationViewUnOptimize);
    }


    /**
     * 条件获取获取光伏预测各时段平均值列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    private List<DynamicQueryPlantPredictedPowerOutput> getAveragePowerGenerationListByPlantIds(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        IPlantPredictedPowerCloudService cloudService = context.getBean(IPlantPredictedPowerCloudService.class);
        XListResult<DynamicQueryPlantPredictedPowerOutput> result = cloudService.queryAveragePowerGenerationListByParam(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 按线路ID集合获取电站列表
     *
     * @param context 上下文
     * @param lineIds 线路ID
     * @return {@link List}<{@link DynamicQueryBasePowerLinePlantOutput}>
     */
    private List<DynamicQueryBasePowerLinePlantOutput> getPowerLinePlantListByLineIds(XContext context, Set<String> lineIds) {
        IBasePowerLineCloudService cloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLinePlantOutput> result = cloudService.queryPowerLinePlantListByParam(context,
                DynamicQueryBasePowerLinePlantInput.builder()
                        .lineIds(lineIds)
                        .build()
        );
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 为间开井口列表设置发电功率
     *
     * @param context           上下文
     * @param spaceWellheadList 空间井口清单
     */
    private void setServiceRatingForSpaceWellheadList(XContext context, List<SpaceInstitutionWellheadView> spaceWellheadList) {
        IBaseWellheadCloudService cloudService = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = cloudService.queryBaseWellheadListByParam(context,
                DynamicQueryBaseWellheadInput.builder()
                        .wellheadIds(spaceWellheadList.stream()
                                .map(SpaceInstitutionWellheadView::getWellheadId)
                                .collect(Collectors.toList())
                        )
                        .build());
        result.throwIfFail();
        List<DynamicQueryBaseWellheadOutput> wellheadOutputList = result.getResult();
        if (CollUtil.isEmpty(wellheadOutputList)) {
            return;
        }
        spaceWellheadList.forEach(s -> {
            s.setServiceRating(
                    wellheadOutputList.stream()
                            .filter(w -> StringUtils.equals(s.getWellheadId(), w.getId()))
                            .findAny()
                            .map(DynamicQueryBaseWellheadOutput::getServiceRating)
                            .orElse(BigDecimal.ZERO)
            );
        });
    }
}